function [ SelectedSubspace ] = OMPSTAR_NN(D,Y,param)
%%---------Description-------------------------------------
% OMP-Star+ is proposed in [1]. It is a generic greedy pursuit algorithm.
% That imposes non-negativity over its solution space.
%   [1] Naveed Akhtar, Faisal Shafait and Ajmal Mian,'Futuristic greedy
%   approach to sparse unmixing of hyperspectral data', IEEE TGRS,2014
% The implementation uses stopping rule (b) and (c)

%%
% Set parameters
% See [1] for details on parameters
perB = param.t;      
err = param.epsilon_0;
f = param.f;
b = param.beta;

SelectedSubspace = [];
% for each pixel in the image
for ii = 1:size(Y,2)
    r = Y(:,ii);                        % consider the pixel as the residue
    S = [];
    while (sum(r.^2)> err)
        %% find the most correlated atoms, using parameter t
        r0 = r;
        proj =(normc(D))'*normc(r);
        [v, i] = sort(proj, 'descend');
        val = v(1,1);
        ind = i(1,1);
        count = 2;
        while count > 0
            vmax = v(1,1);
            if v(count,1) > (perB*vmax)
                val = [val v(count,1)];
                ind = [ind i(count,1)];
                count = count+1;
            else
                count = 0;
            end
        end
        indTest = ind;
        %% From the correlated atoms select the best through ForwardSelection       
        GoodAtomInd = indTest;
        if size(GoodAtomInd,2)>1
            s = potentials(D, Y(:,ii), S, GoodAtomInd, f);
        else
            s = GoodAtomInd;
        end
        %% Augment the current selected subspace
        S = [S s];
        %% Update residual        
        a = lsqnonneg(D(:,S), Y(:,ii));
        r = Y(:,ii) - (D(:,S)*a);
        %% Stopping rule        
        if sum(r.^2) > b *sum(r0.^2)
            break;
        end
        
    end
    SelectedSubspace = [SelectedSubspace S'];
end
end

function [best] = potentials(D, y, preS, G,f)
%%Selects the best atom after looking f steps forward
p = f;          
Scores = [];
for ii = 1:size(G, 2)
    S = [preS G(1,ii)];
    a = lsqnonneg(D(:,S), y);
    resPre = y - D(:,S)*a;
    r = resPre;
    
    rfuture = [];
    for i = 1:p
        [v nextbest] = max(((normc(D))' * normc(r)));
        S = [S nextbest];
        a = lsqnonneg(D(:,S), y);
        r = y-D(:,S)*a;
        rfuture = [rfuture r];
    end
    rfuture = sum(sum(rfuture.^2));     
    score = (rfuture+(sum(resPre.^2))); 
    Scores = [Scores score];
end
[v i] = min(Scores);
best = G(1,i);
end










