%% demo_OMP-Star [1]
%[1] Naveed Akhtar, Faisal Shafait and Ajmal Mian,'Futuristic greedy
%approach to sparse unmixing of hyperspectral data', IEEE TGRS,2014
%
%%----------To run-------------------------------------------------
% Add the folder to Matlab path and run this file
%
%%----------Description--------------------------------------------
%   OMP-Star finds approximate solution to the following l_0 problem
%   min_a ||a||_0 s.t. ||Da - y||_2 =< epsilon_0
%   The non-negative variant OMP-Star+ solves the following constrained optimization problem
%   min_a ||a||_0 s.t. ||Da - y||_2 =< epsilon_0 & a>=0
%   In above:
%       - D in R^(m x k): dictionary/spectral library (remains unchanged)
%       - a in R^(k x 1): sparse code vector/fractional abundances
%       - y in R^(m x 1): input signal/mixed hyperspectral pixel
%
%%----------NOTES--------------------------------------------------
%   In [1] we showed that due to non-negativity the sum-to-one constraint is
%   automatically satisfied by the proposed approach by normalizing the
%   dictioanry in l_1 norm. Therefore, no explicit sum-to-one constraint is
%   used for hyperspectral unmixing.
%
%Author: Naveed Akhtar, 2014
%___________________________________________________________________

%%-----------------Parameter Settings ------------------------
pixels = 500;           % Number of pixels in the synthetic image
p = 3;                  % Cardinality or sparsity of the mixed pixel
positivity = 'false';    % or 'true': OMP-Star if false, OMP-Star+ if true
s = 5;                  % band difference for the derivative

%Algorithm parameters
param.epsilon_0 = 0.001;% for stopping rule (b). This is per pixel err here
param.beta = 0.9;       % for stopping rule (c)
param.t = 0.92;         % algorithm parameter t in Section IV-A
param.f = 2;            % algorithm parameter f in Section IV-A, forward selection

d = load('dictionary');
D = d.D;
[m, k] = size(D);

[Y,A, Aval] = createImage(D, p, pixels);
D = D./repmat(sum(D),m,1);

Yd = specDerivative(Y,s);
Dd = specDerivative(D,s);

if strcmp(positivity, 'false')
    AOMPS = [];
    erecovery_OMPS = 0;
    UnmixingErrOMPS = [];
    tic
    Al2 = [];
    % Unmixing is performed pixel-by-pixel
    for i = 1:size(Yd, 2)    
        [A_OMPSTAR] = OMPSTAR(Dd,Yd(:,i),param); 
        UE = analyzeUnmixErr(A_OMPSTAR, Aval(:,i), D, Y(:,i));    
        eOMPS = supportAnalysis(A_OMPSTAR, A(:,i));
        erecovery_OMPS = erecovery_OMPS + eOMPS;
        UnmixingErrOMPS = [UnmixingErrOMPS UE];
        Al2 = [Al2 (Aval(:,i)'*Aval(:,i))];
    end
    timer = toc;
    OMPStar_Recovery = erecovery_OMPS/size(Y, 2);
    UF_OMPStar = (OMPStar_Recovery(1,1)/p);
    Mean_UE_OMPStar = sum(UnmixingErrOMPS)/ size(UnmixingErrOMPS,2);
    disp(['Unmixing fidelity: ', num2str(UF_OMPStar), ', Mean Euclidean distance:', num2str(Mean_UE_OMPStar), ', Time taken:', num2str(timer)])
    
elseif strcmp(positivity, 'true')
    AOMPSnn = [];
    erecovery_OMPSnn = 0;
    UEOMPSnn = [];
    tic
    Al2 = [];
    % Unmixing is performed pixel-by-pixel
    for i = 1:size(Y, 2)
        A_OMPSTARnn = OMPSTAR_NN(Dd,Yd(:,i),param);
        ue = analyzeUnmixErr(A_OMPSTARnn, Aval(:,i), D, Y(:,i));
        eOMPSnn = supportAnalysis(A_OMPSTARnn, A(:,i));
        erecovery_OMPSnn = erecovery_OMPSnn + eOMPSnn;
        UEOMPSnn = [UEOMPSnn ue];
        Al2 = [Al2 (Aval(:,i)'*Aval(:,i))];
    end
    timer = toc;
    OMPStarNN_Recovery = erecovery_OMPSnn/size(Y, 2);
    UF_OMPSTARNN = (OMPStarNN_Recovery(1,1)/p);
    Mean_UE_OMPStarNN = sum(UEOMPSnn)/size(UEOMPSnn,2);
    disp(['Unmixing fidelity: ', num2str(UF_OMPSTARNN), ', Mean Euclidean distance:', num2str(Mean_UE_OMPStarNN), ', Time taken:', num2str(timer)])
else
    disp('Please enter the correct value for "positivity" parameter...')
end
